@@ -253,7 +253,9 @@ extern "C"
 #endif
 
 #if !defined(MZ_NORETURN)
-# if defined(__GNUC__) || defined(__clang__)
+# if !defined(MZ_PRECISE_RETURN_SPEC)
+#  define MZ_NORETURN
+# elif defined(__GNUC__) || defined(__clang__)
 #  define MZ_NORETURN __attribute__((noreturn))
 # elif defined(_MSC_VER)
 #  define MZ_NORETURN __declspec(noreturn)
@@ -951,7 +953,8 @@ enum {
   SCHEME_hash_string,
   SCHEME_hash_ptr,
   SCHEME_hash_weak_ptr,
-  SCHEME_hash_late_weak_ptr
+  SCHEME_hash_late_weak_ptr,
+  SCHEME_hash_ephemeron_ptr
 };
 
 enum {
@@ -1699,8 +1702,11 @@ MZ_EXTERN void scheme_jit_setjmp_prepare(mz_jit_jmp_buf b);
                                GC_variable_stack = (void **)(void*)(b).gcvs, \
                                scheme_jit_longjmp((b).jb, v))
 # define scheme_setjmp(b)     ((b).gcvs = (intptr_t)__gc_var_stack__, \
-                               (b).gcvs_cnt = (intptr_t)(__gc_var_stack__[1]), \
+                               (b).gcvs_cnt = XFORM_CURRENT_COUNT, \
                                scheme_jit_setjmp((b).jb))
+# ifndef MZ_XFORM
+#  define XFORM_CURRENT_COUNT (intptr_t)(__gc_var_stack__[1])
+# endif
 #else
 # define scheme_longjmp(b, v) scheme_jit_longjmp(b, v)
 # define scheme_setjmp(b) scheme_jit_setjmp(b)
@@ -1933,7 +1939,10 @@ MZ_EXTERN void scheme_set_compiled_file_roots(Scheme_Object *list);
 MZ_EXTERN void scheme_set_dll_path(wchar_t *s);
 typedef void *(*scheme_dll_open_proc)(const char *name, int as_global);
 typedef void *(*scheme_dll_find_object_proc)(void *h, const char *name);
-MZ_EXTERN void scheme_set_dll_procs(scheme_dll_open_proc, scheme_dll_find_object_proc);
+typedef void (*scheme_dll_close_proc)(void *h);
+MZ_EXTERN void scheme_set_dll_procs(scheme_dll_open_proc,
+                                    scheme_dll_find_object_proc,
+                                    scheme_dll_close_proc);
 #endif
 
 MZ_EXTERN void scheme_init_collection_paths(Scheme_Env *global_env, Scheme_Object *extra_dirs);
@@ -1994,6 +2003,8 @@ MZ_EXTERN Scheme_Object *scheme_param_config2(char *name, Scheme_Object *pos,
                                               int isbool);
 MZ_EXTERN Scheme_Object *scheme_register_parameter(Scheme_Prim *function, char *name, int which);
 
+MZ_EXTERN void scheme_set_default_locale(void);
+
 #endif /* SCHEME_DIRECT_EMBEDDED */
 
 /*========================================================================*/
@@ -2092,6 +2103,7 @@ extern Scheme_Extension_Table *scheme_extension_table;
 #define SCHEME_STRUCT_EXPTIME 0x80
 #define SCHEME_STRUCT_NO_MAKE_PREFIX 0x100
 #define SCHEME_STRUCT_NAMES_ARE_STRINGS 0x200
+#define SCHEME_STRUCT_BUILTIN 0x400
 
 /*========================================================================*/
 /*                           file descriptors                             */
